package xslavic4.plan_vyroby;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

public class Vyroba implements Serializable{

	private static final long serialVersionUID = 8680519686374689315L;
	protected static ArrayList<Stroj> vyrobniStroje = new ArrayList<Stroj>();
	protected static ArrayList<Zamestnanec> zamestnanci = new ArrayList<Zamestnanec>();
	protected static ArrayList<Material> material = new ArrayList<Material>();
	protected static ArrayList<Oznameni> oznameni = new ArrayList<Oznameni>();

	public Vyroba() {
	}

	public static Stroj vytvoritStroj(Zamestnanec mistr, String sap, int cs) throws Exception {
		if(strojExistuje(cs, sap)){
			Prihlaseni.zobrazChybu("Stroj se stejnm SAP oznaenm nebo slem ji existuje v systmu!");
			throw new Exception("Stroj se stejnym SAP oznacenim nebo cislem jiz existuje v systemu!");
		}else{
			Stroj st = new Stroj(mistr);
			Vyroba.vyrobniStroje.add(st);
			st.setSAPOznaceni(sap);
			st.setCisloStroje(cs);
			return st;
		}
		
	}

	public static void vyraditStroj(Stroj stroj) {
		Vyroba.vyrobniStroje.remove(stroj);
	}

	public static ArrayList<Zamestnanec> getZamestnanci() {
		return zamestnanci;
	}

	public static void addZamestnanci(int cz, String log, String rol,String jm, String pr) throws Exception {
		zamestnanci.add(new Zamestnanec(cz,log,rol,jm,pr));
	}

	public static void remZamestnanci(int idx) {
		if(idx==0){
			try {
				Prihlaseni.zobrazChybu("Zamstnance s tmto ID nelze zruit!");
				throw new Exception("Zamestnance s timto ID nelze zrusit!");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}else{
			zamestnanci.remove(idx);
		}
	}

	//koeficient jednoho stroje daneho tydne
	public Double koeficient(Stroj stroj, Calendar zacatekTydne) {
		Calendar konecTydne = (Calendar) zacatekTydne.clone();
		konecTydne.add(Calendar.DATE, 7);
		return koeficient(stroj, zacatekTydne, konecTydne);
	}

	//koeficient vsech stroju daneho tydne
	public Double koeficient(Calendar zacatekTydne) {
		Calendar konecTydne = (Calendar) zacatekTydne.clone();
		konecTydne.add(Calendar.DATE, 7);
		return koeficient(zacatekTydne, konecTydne);
	}

	//koeficient jednoho stroje v rozmezi libovolnych dnu
	public Double koeficient(Stroj stroj, Calendar odData, Calendar doData) {
		double vyrobeno = 0;
		double dilec = 0;
		for(int i = 0; i < stroj.getDavka().size(); i++){
			if(stroj.getDavka().get(i).getPlanovanyKonec().getTimeInMillis()>=odData.getTimeInMillis()){
				dilec += stroj.getDavka().get(i).getMaterial().getPocet();
				for(int x = 0; x < stroj.getDavka().get(i).getDataSmen().size();x++){
					if(stroj.getDavka().get(i).getDataSmen().get(x).getTimeInMillis()>=odData.getTimeInMillis() && stroj.getDavka().get(i).getDataSmen().get(x).getTimeInMillis()<doData.getTimeInMillis()){
						try{
							vyrobeno += stroj.getDavka().get(i).getSkutecnostSmen().get(x);
						}catch(Exception ex){
						}
					}
					if(stroj.getDavka().get(i).getDataSmen().get(x).getTimeInMillis()>=doData.getTimeInMillis()){
						try{
							return vyrobeno / dilec;
						}catch(Exception ex){
							return 0.0;
						}
					}
					if(x == stroj.getDavka().get(i).getDataSmen().size()-1){
						i++;
						x=-1;
						if(stroj.getDavka().get(i).getDataSmen().get(0).getTimeInMillis()<doData.getTimeInMillis()){
							dilec += stroj.getDavka().get(i).getMaterial().getPocet();
						}
					}
				}
				try{
					return vyrobeno / dilec;
				}catch(Exception ex){
					return 0.0;
				}
			}
		}
		return 0.0;
	}

	//koeficient vsech stroju v rozmezi libovolnych dnu
	public Double koeficient(Calendar odData, Calendar doData) {
		double koef = 0;
		for(int i = 0; i < vyrobniStroje.size(); i++){
			koef += koeficient(vyrobniStroje.get(i), odData, doData);
		}
		return koef;
	}

	public static boolean strojExistuje(int cisStroje, String sap) {
		for(int i = 0; i < vyrobniStroje.size(); i++){
			if(vyrobniStroje.get(i).getCisloStroje()==cisStroje || vyrobniStroje.get(i).getSAPOznaceni().equals(sap)){
				return true;
			}
		}
		return false;
	}

	public static boolean jeLoginJedinecny(String login) {
		for(int i = 0; i < Vyroba.zamestnanci.size(); i++){
			if(Vyroba.zamestnanci.get(i).getLogin().equals(login)){
				return false;
			}
		}
		return true;
	}

	public static void zrusitPrideleniZamestnance(Zamestnanec zam) throws Exception {
		for(int i = 0; i < vyrobniStroje.size();i++){
			for(int x = 0; x < vyrobniStroje.get(i).getMistri().size(); x++){
				if(vyrobniStroje.get(i).getMistri().get(x).getCisloZamestnance()==zam.getCisloZamestnance()){
					try{
						vyrobniStroje.get(i).remMistri(x);
						x--;
						vyrobniStroje.get(i).getMistri().get(0);
					}catch(Exception ex){
						// Hlaska o tom, ze by stroj jinak nemel mistra - vypsat oznaceni stroje uzivateli
						Prihlaseni.zobrazChybu("Nelze zruit piazen. Stroj by neml dnho mistra. Nejprve piate ke stroji " + vyrobniStroje.get(i).getCisloStroje() + " jinho mistra.");
						throw new Exception("Nelze zruit piazen. Stroj by neml dnho mistra.");
					}
				}
			}
			for(int x = 0; x < vyrobniStroje.get(i).getDelnici().size(); x++){
				if(vyrobniStroje.get(i).getDelnici().get(x).getCisloZamestnance()==zam.getCisloZamestnance()){
					vyrobniStroje.get(i).remDelnici(x);
					x--;
				}
			}
		}
	}

	public static void setZamestnanci(ArrayList<Zamestnanec> zam){
		zamestnanci = zam;
	}
	
	public static void setVyrobniStroje(ArrayList<Stroj> str){
		vyrobniStroje = str;
	}
	
	public static void addMaterial(String sap, int poc) throws Exception {
		Vyroba.material.add(new Material(sap, poc));
	}
	
	public static List<Material> getMaterial() {
		return Vyroba.material;
	}
}